@@ -1234,20 +1234,28 @@ simdutf_warn_unused size_t implementation::utf32_length_from_utf8(
 simdutf_warn_unused result implementation::base64_to_binary(
     const char *input, size_t length, char *output, base64_options options,
     last_chunk_handling_options last_chunk_options) const noexcept {
-  if (options & base64_url) {
-    if (options == base64_options::base64_url_accept_garbage) {
-      return base64::compress_decode_base64<true, true>(
+  if (options & base64_default_or_url) {
+    if (options == base64_options::base64_default_or_url_accept_garbage) {
+      return base64::compress_decode_base64<false, true, true>(
           output, input, length, options, last_chunk_options);
     } else {
-      return base64::compress_decode_base64<true, false>(
+      return base64::compress_decode_base64<false, false, true>(
+          output, input, length, options, last_chunk_options);
+    }
+  } else if (options & base64_url) {
+    if (options == base64_options::base64_url_accept_garbage) {
+      return base64::compress_decode_base64<true, true, false>(
+          output, input, length, options, last_chunk_options);
+    } else {
+      return base64::compress_decode_base64<true, false, false>(
           output, input, length, options, last_chunk_options);
     }
   } else {
     if (options == base64_options::base64_default_accept_garbage) {
-      return base64::compress_decode_base64<false, true>(
+      return base64::compress_decode_base64<false, true, false>(
           output, input, length, options, last_chunk_options);
     } else {
-      return base64::compress_decode_base64<false, false>(
+      return base64::compress_decode_base64<false, false, false>(
           output, input, length, options, last_chunk_options);
     }
   }
@@ -1256,20 +1264,28 @@ simdutf_warn_unused result implementation::base64_to_binary(
 simdutf_warn_unused full_result implementation::base64_to_binary_details(
     const char *input, size_t length, char *output, base64_options options,
     last_chunk_handling_options last_chunk_options) const noexcept {
-  if (options & base64_url) {
-    if (options == base64_options::base64_url_accept_garbage) {
-      return base64::compress_decode_base64<true, true>(
+  if (options & base64_default_or_url) {
+    if (options == base64_options::base64_default_or_url_accept_garbage) {
+      return base64::compress_decode_base64<false, true, true>(
           output, input, length, options, last_chunk_options);
     } else {
-      return base64::compress_decode_base64<true, false>(
+      return base64::compress_decode_base64<false, false, true>(
+          output, input, length, options, last_chunk_options);
+    }
+  } else if (options & base64_url) {
+    if (options == base64_options::base64_url_accept_garbage) {
+      return base64::compress_decode_base64<true, true, false>(
+          output, input, length, options, last_chunk_options);
+    } else {
+      return base64::compress_decode_base64<true, false, false>(
           output, input, length, options, last_chunk_options);
     }
   } else {
     if (options == base64_options::base64_default_accept_garbage) {
-      return base64::compress_decode_base64<false, true>(
+      return base64::compress_decode_base64<false, true, false>(
           output, input, length, options, last_chunk_options);
     } else {
-      return base64::compress_decode_base64<false, false>(
+      return base64::compress_decode_base64<false, false, false>(
           output, input, length, options, last_chunk_options);
     }
   }
@@ -1278,20 +1294,28 @@ simdutf_warn_unused full_result implementation::base64_to_binary_details(
 simdutf_warn_unused result implementation::base64_to_binary(
     const char16_t *input, size_t length, char *output, base64_options options,
     last_chunk_handling_options last_chunk_options) const noexcept {
-  if (options & base64_url) {
-    if (options == base64_options::base64_url_accept_garbage) {
-      return base64::compress_decode_base64<true, true>(
+  if (options & base64_default_or_url) {
+    if (options == base64_options::base64_default_or_url_accept_garbage) {
+      return base64::compress_decode_base64<false, true, true>(
           output, input, length, options, last_chunk_options);
     } else {
-      return base64::compress_decode_base64<true, false>(
+      return base64::compress_decode_base64<false, false, true>(
+          output, input, length, options, last_chunk_options);
+    }
+  } else if (options & base64_url) {
+    if (options == base64_options::base64_url_accept_garbage) {
+      return base64::compress_decode_base64<true, true, false>(
+          output, input, length, options, last_chunk_options);
+    } else {
+      return base64::compress_decode_base64<true, false, false>(
           output, input, length, options, last_chunk_options);
     }
   } else {
     if (options == base64_options::base64_default_accept_garbage) {
-      return base64::compress_decode_base64<false, true>(
+      return base64::compress_decode_base64<false, true, false>(
           output, input, length, options, last_chunk_options);
     } else {
-      return base64::compress_decode_base64<false, false>(
+      return base64::compress_decode_base64<false, false, false>(
           output, input, length, options, last_chunk_options);
     }
   }
@@ -1300,20 +1324,28 @@ simdutf_warn_unused result implementation::base64_to_binary(
 simdutf_warn_unused full_result implementation::base64_to_binary_details(
     const char16_t *input, size_t length, char *output, base64_options options,
     last_chunk_handling_options last_chunk_options) const noexcept {
-  if (options & base64_url) {
-    if (options == base64_options::base64_url_accept_garbage) {
-      return base64::compress_decode_base64<true, true>(
+  if (options & base64_default_or_url) {
+    if (options == base64_options::base64_default_or_url_accept_garbage) {
+      return base64::compress_decode_base64<false, true, true>(
           output, input, length, options, last_chunk_options);
     } else {
-      return base64::compress_decode_base64<true, false>(
+      return base64::compress_decode_base64<false, false, true>(
+          output, input, length, options, last_chunk_options);
+    }
+  } else if (options & base64_url) {
+    if (options == base64_options::base64_url_accept_garbage) {
+      return base64::compress_decode_base64<true, true, false>(
+          output, input, length, options, last_chunk_options);
+    } else {
+      return base64::compress_decode_base64<true, false, false>(
           output, input, length, options, last_chunk_options);
     }
   } else {
     if (options == base64_options::base64_default_accept_garbage) {
-      return base64::compress_decode_base64<false, true>(
+      return base64::compress_decode_base64<false, true, false>(
           output, input, length, options, last_chunk_options);
     } else {
-      return base64::compress_decode_base64<false, false>(
+      return base64::compress_decode_base64<false, false, false>(
           output, input, length, options, last_chunk_options);
     }
   }
